﻿//------------------------------------------------------------
//    AIManager.cs - Keeps track of AINurses - manages shifts.
//
//    Written by Karn Bianco
//------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TotallyRadEngine10;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace ProjectMaternity
{
    public class AIManager
    {
        public List<AINurse> mAINurses;

        private List<Patient> mPatients;
        private List<Bed> mBeds;
        private List<Cabinet> mCabinets;

        private int mNursesOnCall = 0;
        private int mOnCallTimer = 0;
        private int mOnCallLevel = 0;
        private int[] mOnCallLevelTimes = new int[] { 100, 200, 300, 400, 500, 600 };    // Extra nurse, extra time and so on...
        private bool mOnCallShiftActive = false;
        private int mShiftTimer = 0;
        private int mMaxShiftTime = 0;

        public bool Enabled = true;

        #region Properties

        public bool OnCallAvailable
        {
            get
            {
                return mOnCallLevel > 0;
            }
        }

        public int OnCallTimer
        {
            get
            {
                return mOnCallTimer;
            }
        }

        public int MaxOnCallTimer
        {
            get
            {
                return mOnCallLevelTimes[mOnCallLevel];
            }
        }

        public int OnCallLevel
        {
            get
            {
                return mOnCallLevel;
            }
        }

        public bool ShiftActive
        {
            get
            {
                return mOnCallShiftActive;
            }
        }

        public int ShiftTimer
        {
            get
            {
                return mShiftTimer;
            }
        }

        public int MaxShiftTimer
        {
            get
            {
                return mMaxShiftTime;
            }
        }

        #endregion

        /// <summary>
        /// Instantiate AIManager
        /// </summary>
        /// <param name="patients">List of level's patients</param>
        /// <param name="beds">List of level's beds</param>
        /// <param name="cabinets">List of level's cabinets</param>
        public AIManager(List<Patient> patients, List<Bed> beds, List<Cabinet> cabinets)
        {
            // Keep references to levle objects
            this.mPatients = AINurse.Patients = patients;
            this.mBeds = AINurse.Beds = beds;
            this.mCabinets = AINurse.Cabinets = cabinets;

            // Initialize AINurses
            mAINurses = new List<AINurse>();
            mAINurses.Add(new AINurse(new Vector3(-2, 0, -5)));
            mAINurses[0].Position.Z = 60;
            mAINurses[0].IsCellShaded = true;
            mAINurses[0].Active = false;

            mAINurses.Add(new AINurse(new Vector3(-2, 0, -10)));
            mAINurses[1].Position.Z = 65;
            mAINurses[1].IsCellShaded = true;
            mAINurses[1].Active = false;

            mAINurses.Add(new AINurse(new Vector3(-2, 0, -15)));
            mAINurses[2].Position.Z = 70;
            mAINurses[2].IsCellShaded = true;
            mAINurses[2].Active = false;
        }

        /// <summary>
        /// Keep track of shifts - get nurses to leave when applicable.
        /// </summary>
        /// <param name="gameTime"></param>
        /// <param name="player"></param>
        public void Update(GameTime gameTime, Player player)
        {
            if (Enabled)
            {
                if (mOnCallShiftActive)
                {
                    if (mShiftTimer < mMaxShiftTime)
                    {
                        mShiftTimer++;
                    }
                    else
                    {
                        mOnCallShiftActive = false;
                        mOnCallTimer = 0;
                        mShiftTimer = 0;
                        mOnCallLevel = 0;

                        // Shift is over, leave the hospital
                        for (int i = 0; i < mNursesOnCall; i++)
                        {
                            mAINurses[i].StopInteractionWith<Bed>();
                            mAINurses[i].StopInteractionWith<Patient>();
                            mAINurses[i].SetLeaveHospital();
                        }

                        mNursesOnCall = 0;
                    }
                }
                else
                {
                    if (mOnCallTimer <= MaxOnCallTimer)
                    {
                        mOnCallTimer++;

                        if (mOnCallTimer >= mOnCallLevelTimes[mOnCallLevel])
                        {
                            if (mOnCallLevel < mOnCallLevelTimes.Length - 1)
                            {
                                mOnCallLevel++;
                            }

                            // Odd levels increase the number of nurses available
                            if (mOnCallLevel % 2 != 0)
                            {
                                if (mNursesOnCall < 3) mNursesOnCall++;
                                mMaxShiftTime = 2000;
                            }
                            // Even levels increase the shift length for that number of nurses
                            else
                            {
                                mMaxShiftTime = 3000;
                            }
                        }
                    }

                    // Call in the AI nurses
                    if ((InputManager.HasButtonBeenPressed(Keys.Enter) || InputManager.HasButtonBeenPressed(Buttons.Y)))
                    {
                        if (mOnCallLevel > 0)
                        {
                            mOnCallShiftActive = true;
                            mShiftTimer = 0;

                            for (int i = 0; i < mNursesOnCall; i++)
                            {
                                mAINurses[i].Active = true;
                                mAINurses[i].SetEnterHospital();
                            }
                        }
                    }
                }

                foreach (AINurse aiNurse in mAINurses)
                {
                    if (aiNurse.Active)
                    {
                        aiNurse.Update(gameTime);
                    }

                    CollisionManager.Resolve(aiNurse, player);
                }
            }
        }
    }
}
