﻿//------------------------------------------------------------
//    AINurse.cs - AI controller for helper nurses.
//    
//    Written by Karn Bianco
//------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using TotallyRadEngine10;

namespace ProjectMaternity
{
    public enum AIGoal
    {
        Idle,
        GetPatient,
        MovePatientToBed,
        AssistPatient,
        ResolveComplication,
        CleanBed,
        EnterHospital,
        LeaveHospital,
    }

    public class AINurse : Nurse
    {
        #region Fields

        // Entitys being targeted (access via properties)
        private Patient mTargetPatient = null;
        private Chair mTargetChair = null;
        private Cabinet mTargetCabinet = null;
        private Bed mTargetBed = null;
        private string mTargetBedInitialState = "";

        // Beds to target (access via properties)
        private Bed mBedComplicated = null;
        private Bed mBedAssist = null;
        private Bed mBedDirty = null;
        private Bed mBedEmpty = null;

        /// <summary>
        /// What is the Nurse attempting to do?
        /// </summary>
        public AIGoal mGoal = AIGoal.Idle;

        #endregion

        private Vector3 mPrevPosition = Vector3.Zero;

        private float mElapsedMillisecs = 0;
        private float mMaxTimeStationary = 3;

        private float mMicroPathFindTime = 0;
        private float mMicroPathFindMaxTime = 2;

        public Color ColorAINurse; // FOR DEBUGGING INNIT

        public bool Active;

        #region Properties

        public static List<Patient> Patients;
        public static List<Bed> Beds;
        public static List<Cabinet> Cabinets;

        protected Vector3 mSpawnPoint;
        public MacroPathFinder MacroPathFinder;
        public MicroPathFinder MicroPathFinder;

        public static bool ResolveComplications = false;

        bool mNoAutoAnim = false;

        // These properties also update the TargetedBy property
        // of the given Entity to be 'null' or 'this'

        protected Patient TargetPatient
        {
            get { return mTargetPatient; }
            set
            {
                if (value == null)
                {
                    if (mTargetPatient.TargetedBy == this)
                        mTargetPatient.TargetedBy = null;

                    mTargetPatient = value;
                }
                else
                {
                    mTargetPatient = value;
                    mTargetPatient.TargetedBy = this;
                }
            }
        }
        protected Chair TargetChair
        {
            get { return mTargetChair; }
            set
            {
                if (value == null && mTargetChair != null)
                {
                    if (mTargetChair.TargetedBy == this)
                        mTargetChair.TargetedBy = null;

                    mTargetChair = value;
                }
                else if (value != null)
                {
                    mTargetChair = value;
                    mTargetChair.TargetedBy = this;
                }
            }
        }
        protected Cabinet TargetCabinet
        {
            get { return mTargetCabinet; }
            set
            {
                if (value == null)
                {
                    mTargetCabinet.TargetedBy = null;
                    mTargetCabinet = value;
                }
                else
                {
                    mTargetCabinet = value;
                    mTargetCabinet.TargetedBy = this;
                }
            }
        }
        protected Bed TargetBed
        {
            get { return mTargetBed; }
            set
            {
                if (value == null)
                {
                    if (mTargetBed.TargetedBy == this)
                        mTargetBed.TargetedBy = null;

                    mTargetBedInitialState = "";
                    mTargetBed = value;
                }
                else
                {
                    mTargetBed = value;
                    mTargetBedInitialState = mTargetBed.CurrentState;
                    mTargetBed.TargetedBy = this;
                }
            }
        }
        protected string TargetBedInitialState
        {
            get { return mTargetBedInitialState; }
        }

        #endregion

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="spawnPoint">Where to spawn the Nurse</param>
        public AINurse(Vector3 spawnPoint)
        {
            MacroPathFinder = new MacroPathFinder(this);
            MicroPathFinder = new MicroPathFinder(this);

            this.Position = mSpawnPoint = spawnPoint;
            this.mPrevPosition = new Vector3(-1, -1, -1);

            Model = ContentCache.GetModel("Models\\Animated\\NurseIdle2");  // load a model for the player
            Scale = new Vector3(0.425f, 0.425f, 0.425f);
            InitialiseAnimation("Content\\Models\\Animated\\NurseAnims.tra");
            SetAnimation("Idle", 0, 1.0f);

            foreach (ModelMesh mesh in Model.Meshes)
            {
                foreach (ModelMeshPart part in mesh.MeshParts)
                {
                    (part.Effect as BasicEffect).Texture = ContentCache.GetTexture("Models\\Animated\\NurseAssistant");
                }
            }
        }

        /// <summary>
        /// Main update loop.
        /// </summary>
        /// <param name="gameTime"></param>
        public override void Update(GameTime gameTime)
        {
            // Animate based on pathfinding
            if ((MacroPathFinder.IsFollowingPath || MicroPathFinder.IsFollowingPath) && mGoal != AIGoal.EnterHospital && !mNoAutoAnim)
            {
                if (IsAnimationPlaying && !CurrentAnimationName.Contains("Run"))
                {
                    if (CurrentItem == Item.None)
                    {
                        SetAnimation("Run", 0.0f, 3.0f);
                    }
                    else
                    {
                        SetAnimation("RunSheets", 0.0f, 3.0f);
                    }
                }
            }
            else if (!(MacroPathFinder.IsFollowingPath || MicroPathFinder.IsFollowingPath) && mGoal != AIGoal.EnterHospital && !mNoAutoAnim && !IsInteractingWith<Bed>())
            {
                if (IsAnimationPlaying && !CurrentAnimationName.Contains("Idle"))
                {
                    if (CurrentItem == Item.None)
                    {
                        SetAnimation("Idle", 0.0f, 1.0f);
                    }
                    else
                    {
                        SetAnimation("IdleSheets", 0.0f, 1.0f);
                    }
                }
            }
            mElapsedMillisecs += (float)(gameTime.ElapsedGameTime.TotalMilliseconds / 1000);

            if (MicroPathFinder.FindingPath && !MicroPathFinder.HasPathBeenFound && !MicroPathFinder.IsFollowingPath)
            {
                mMicroPathFindTime += mElapsedMillisecs;
            }

            if (mElapsedMillisecs >= mMaxTimeStationary)
            {
                // Nurse has been standing still too long, do something!
                if (mPrevPosition == Position && mGoal != AIGoal.Idle)
                {
                    switch (mGoal)
                    {
                        case AIGoal.GetPatient:
                            SetIdle();
                            break;
                        case AIGoal.MovePatientToBed:
                            SetMovePatientToBed(TargetBed);
                            break;
                        case AIGoal.AssistPatient:
                            SetAssistPatient(TargetBed);
                            break;
                        case AIGoal.ResolveComplication:
                            SetResolveComplication(TargetBed);
                            break;
                        case AIGoal.CleanBed:
                            SetCleanBed(TargetBed);
                            break;
                        case AIGoal.EnterHospital:
                            SetEnterHospital();
                            break;
                        case AIGoal.LeaveHospital:
                            SetLeaveHospital();
                            break;
                        default:
                            SetIdle();
                            break;
                    }
                }

                mPrevPosition = Position;
                mElapsedMillisecs = 0;
            }

            if (!IsInteractingWith<Bed>())
            {
                MacroPathFinder.Update(gameTime);
                MicroPathFinder.Update(gameTime);

                switch (mGoal)
                {
                    case AIGoal.Idle:
                        UpdateIdle();
                        break;
                    case AIGoal.GetPatient:
                        UpdateGetPatient();
                        break;
                    case AIGoal.MovePatientToBed:
                        UpdateMovePatientToBed();
                        break;
                    case AIGoal.ResolveComplication:
                        UpdateResolveComplication();
                        break;
                    case AIGoal.AssistPatient:
                        UpdateAssistPatient();
                        break;
                    case AIGoal.CleanBed:
                        UpdateCleanBed();
                        break;
                    case AIGoal.EnterHospital:
                        UpdateEnterHospital();
                        break;
                    case AIGoal.LeaveHospital:
                        UpdateLeaveHospital();
                        break;
                }
            }

            base.Update(gameTime);
        }

        /// Idle
        private void SetIdle()
        {
            // Reset target Entitys
            if (TargetPatient != null) TargetPatient = null;
            if (TargetChair != null) TargetChair = null;
            if (TargetBed != null) TargetBed = null;
            if (TargetCabinet != null) TargetCabinet = null;

            // Set new goal as idle
            mGoal = AIGoal.Idle;

            // Release any patients following
            if (IsInteractingWith<Patient>())
            {
                (GetInteractable<Patient>() as Patient).PickAChair();
                StopInteractionWith<Patient>();
            }

            // Stop interacting with any beds
            if (IsInteractingWith<Bed>())
            {
                StopInteractionWith<Bed>();
            }

            // Cancel current paths
            MacroPathFinder.CancelCurrentPath();
            MicroPathFinder.CancelCurrentPath();
        }
        private void UpdateIdle()
        {
            // Move to spawn point
            if (Position != mSpawnPoint)
                PathFindToPosition(mSpawnPoint);

            // Check for patient-filled, empty, and dirty beds
            CheckBeds();

            // Complicate patients are most urgent
            if (mBedComplicated != null && ResolveComplications)
            {
                SetResolveComplication(mBedComplicated);
            }

            // Next up are delivering patients
            else if (mBedAssist != null)
            {
                SetAssistPatient(mBedAssist);
            }

            // Next up are dirty beds
            else if (mBedDirty != null)
            {
                SetCleanBed(mBedDirty);
            }

            // Finally, empty beds (to place patients in)
            else if (mBedEmpty != null)
            {
                Patient mostUrgentPatient = new Patient();
                mostUrgentPatient.UrgencyTimer = float.MinValue;

                // Find a patient that is sitting down
                for (int p = 0; p < Patients.Count; p++)
                {
                    Patient patient = Patients[p];

                    // If the patient is untargeted/more urgent, store a reference
                    if (patient.UrgencyTimer > mostUrgentPatient.UrgencyTimer &&
                        patient.TargetedBy == null && patient.IsInteractingWith<Chair>())
                    {
                        mostUrgentPatient = patient;
                    }
                }

                // If a suitable patient has been found, go get her
                if (mostUrgentPatient.UrgencyTimer != float.MinValue)
                {
                    SetGetPatient(mostUrgentPatient);
                    TargetBed = mBedEmpty;
                }
            }
        }

        /// Check state of beds
        private void CheckBeds()
        {
            // Reset bed variables
            if (mBedComplicated != null) mBedComplicated = null;
            if (mBedAssist != null) mBedAssist = null;
            if (mBedDirty != null) mBedDirty = null;
            if (mBedEmpty != null) mBedEmpty = null;

            // Check for patient-filled, empty, and dirty beds
            foreach (Bed bed in Beds)
            {
                // Check beds not being targeted/interacted with
                if ((bed.TargetedBy == null || bed.TargetedBy == this) &&
                    !bed.IsInteractingWith<Nurse>())
                {
                    // Is the bed's patient complicated?
                    if (bed.PatientCurrentState == "Complicated" && ResolveComplications)
                    {
                        if (mBedComplicated != null)
                        {
                            // If patient in this bed is more urgent, store reference
                            if (mBedComplicated.PatientUrgencyTimer < bed.PatientUrgencyTimer)
                            {
                                mBedComplicated = bed;
                            }
                        }
                        else
                        {
                            mBedComplicated = bed;
                        }
                    }

                    // Does the bed's patient need assistance?
                    else if (bed.PatientCurrentState == "Delivering")
                    {
                        if (mBedAssist != null)
                        {
                            // If patient in this bed is more urgent, store reference
                            if (mBedAssist.PatientUrgencyTimer < bed.PatientUrgencyTimer)
                            {
                                mBedAssist = bed;
                            }
                        }
                        else
                        {
                            mBedAssist = bed;
                        }
                    }

                    // Is the bed dirty?
                    else if (bed.CurrentState == "Dirty" && mBedDirty == null)
                    {
                        mBedDirty = bed;
                    }

                    // Is the bed empty/clean?
                    else if (bed.CurrentState == "Clean" && mBedEmpty == null)
                    {
                        mBedEmpty = bed;
                    }
                }
            }
        }

        /// Getting patient from chair
        private void SetGetPatient(Patient targetPatient)
        {
            // Store target patient, update her
            TargetPatient = targetPatient;
            TargetChair = targetPatient.GetInteractable<Chair>() as Chair;
            mGoal = AIGoal.GetPatient;

            // Cancel current paths
            MacroPathFinder.CancelCurrentPath();
            MicroPathFinder.CancelCurrentPath();
        }
        private void UpdateGetPatient()
        {
            // Move to chair patient is sitting in, or patient
            if (TargetChair != null)
            {
                PathFindToPosition(TargetChair.Position);

                // Close enough to interact with patient
                if (InteractionPoly.Intersects(ref TargetChair.BoundingPolygon))
                {
                    InteractWith(TargetPatient);
                    SetMovePatientToBed(TargetBed);
                }
            }
            else // Move directly to patient instead
            {
                PathFindToPosition(TargetPatient.Position);

                // Close enough to interact with patient
                if (InteractionPoly.Intersects(ref TargetPatient.BoundingPolygon))
                {
                    InteractWith(TargetPatient);
                    SetMovePatientToBed(TargetBed);
                }
            }

            // Target patient has been hijacked, go idle
            if (TargetPatient.IsInteractingWith<Nurse>() &&
                TargetPatient.GetInteractable<Nurse>() != this)
            {
                SetIdle();
            }

            bool availableBed = false;

            // Ensure there is still an empty bed for the patient
            foreach (Bed bed in Beds)
            {
                if (!bed.IsInteractingWith<Patient>() &&
                    (bed.TargetedBy == null || bed.TargetedBy == this) && bed.CurrentState == "Clean")
                {
                    availableBed = true;
                    break;
                }
            }

            // No available beds left, go idle
            if (!availableBed) { SetIdle(); }
        }

        /// Escort patient to empty bed
        private void SetMovePatientToBed(Bed targetBed)
        {
            // Store target bed
            TargetBed = targetBed;
            mGoal = AIGoal.MovePatientToBed;

            // Cancel current paths
            MacroPathFinder.CancelCurrentPath();
            MicroPathFinder.CancelCurrentPath();
        }
        private void UpdateMovePatientToBed()
        {
            // Move to target bed
            PathFindToPosition(TargetBed.Position);

            // Close enough to interact with the bed
            if (InteractionPoly.Intersects(ref TargetBed.BoundingPolygon)
                && TargetBed.CurrentState != "Dirty")
            {
                InteractWith(TargetBed);    // Put patient in bed
                SetIdle();                  // Return to idle
            }

            if (TargetBed != null)
            {
                // Target bed has changed state OR there is a new patient in the bed
                if (TargetBed.CurrentState != "Clean" ||
                    (TargetBed.IsInteractingWith<Patient>() && this.IsInteractingWith<Patient>()))
                {
                    TargetBed = null;

                    // Find a new target bed
                    foreach (Bed bed in Beds)
                    {
                        if (!bed.IsInteractingWith<Patient>() && bed.CurrentState == "Clean" &&
                            (bed.TargetedBy == null || bed.TargetedBy == this))
                        {
                            SetMovePatientToBed(bed);
                            break;
                        }
                    }

                    // No suitable bed could be found, release patient
                    if (TargetBed == null)
                    {
                        (this.GetInteractable<Patient>() as Patient).PickAChair();
                        StopInteractionWith<Patient>();
                    }
                }
            }

            // AINurse is no longer being followed by a patient
            if (!(this.IsInteractingWith<Patient>()))
            {
                SetIdle();
            }
        }

        /// Assist patient in bed
        private void SetAssistPatient(Bed targetBed)
        {
            // Store target bed
            TargetBed = targetBed;
            TargetCabinet = targetBed.NearestCabMed;
            mGoal = AIGoal.AssistPatient;

            // Cancel current paths
            MacroPathFinder.CancelCurrentPath();
            MicroPathFinder.CancelCurrentPath();
        }
        private void UpdateAssistPatient()
        {
            if (TargetBed != null)
            {
                // Move to bed if targeted
                PathFindToPosition(TargetBed.Position);

                // Close enough to interact with the bed
                if (InteractionPoly.Intersects(ref TargetBed.BoundingPolygon))
                {
                    MacroPathFinder.CancelCurrentPath();
                    MicroPathFinder.CancelCurrentPath();

                    // Assist patient
                    InteractWith(TargetBed);
                }

                // Target is no longer delivering, or is being assisted by some else
                if ((TargetBed.PatientCurrentState != "Delivering" ||
                    (TargetBed.IsInteractingWith<Nurse>() && (TargetBed.GetInteractable<Nurse>() != this))))
                {
                    SetIdle();
                }
            }
        }

        /// Resolve medical complication
        private void SetResolveComplication(Bed targetBed)
        {
            // Store target bed
            TargetBed = targetBed;
            TargetCabinet = targetBed.NearestCabMed;
            mGoal = AIGoal.ResolveComplication;

            // Cancel current paths
            MacroPathFinder.CancelCurrentPath();
            MicroPathFinder.CancelCurrentPath();
        }
        private void UpdateResolveComplication()
        {
            switch (CurrentItem)
            {
                case Item.None:
                    {
                        // Move to medicine nearest bed 
                        PathFindToPosition(TargetCabinet.Position);

                        // Close enough to interact with the cabinet
                        if (InteractionPoly.Intersects(ref TargetCabinet.BoundingPolygon))
                        {
                            MacroPathFinder.CancelCurrentPath();
                            MicroPathFinder.CancelCurrentPath();

                            // Pick up medical equipment
                            InteractWith(TargetCabinet);
                        }

                        // Target is no longer complicated, or is being assisted
                        if ((TargetBed.PatientCurrentState != "Complicated" ||
                            (TargetBed.IsInteractingWith<Nurse>() && (TargetBed.GetInteractable<Nurse>() != this))))
                        {
                            SetIdle();
                        }

                        break;
                    }
                case Item.Medical:
                    {
                        if (TargetBed != null)
                        {
                            // Move to bed if targeted
                            PathFindToPosition(TargetBed.Position);

                            // Close enough to interact with the bed
                            if (InteractionPoly.Intersects(ref TargetBed.BoundingPolygon))
                            {
                                MacroPathFinder.CancelCurrentPath();
                                MicroPathFinder.CancelCurrentPath();

                                // Resolve complication
                                InteractWith(TargetBed);
                            }

                            // Target is no longer complicated, or is being assisted
                            if ((TargetBed.PatientCurrentState != "Complicated" ||
                                (TargetBed.IsInteractingWith<Nurse>() && (TargetBed.GetInteractable<Nurse>() != this))))
                            {
                                TargetCabinet = TargetBed.NearestCabMed;
                                TargetBed = null;
                            }
                        }

                        // Bed no longer being targeted, but still have medical equipment
                        if (TargetBed == null && CurrentItem == Item.Medical)
                        {
                            // Cancel previous path to target bed
                            if (MacroPathFinder.TargetPosition2D != MacroPathFinder.GetCollisionMapNode(
                                new Vector2(TargetCabinet.Position.X, TargetCabinet.Position.Z)).Position)
                            {
                                MacroPathFinder.CancelCurrentPath();
                                MicroPathFinder.CancelCurrentPath();
                            }

                            // Move to nearest medical cabinet
                            PathFindToPosition(TargetCabinet.Position);

                            // Close enough to interact with the cabinet
                            if (InteractionPoly.Intersects(ref TargetCabinet.BoundingPolygon))
                            {
                                MacroPathFinder.CancelCurrentPath();
                                MicroPathFinder.CancelCurrentPath();

                                // Return medical equipment
                                InteractWith(TargetCabinet);
                                CurrentItem = Item.None; // Prevents odd error
                                SetIdle();
                            }
                        }
                        break;
                    }
            }
        }

        /// Clean dirty bed
        private void SetCleanBed(Bed targetBed)
        {
            // Store target bed
            TargetBed = targetBed;
            TargetCabinet = targetBed.NearestCabLaundry;
            mGoal = AIGoal.CleanBed;

            // Cancel current paths
            MacroPathFinder.CancelCurrentPath();
            MicroPathFinder.CancelCurrentPath();
        }
        private void UpdateCleanBed()
        {
            switch (CurrentItem)
            {
                case Item.None:
                    {
                        // Move to laundry cabinet nearest bed 
                        PathFindToPosition(TargetCabinet.Position);

                        // Close enough to interact with the cabinet
                        if (InteractionPoly.Intersects(ref TargetCabinet.BoundingPolygon))
                        {
                            MacroPathFinder.CancelCurrentPath();
                            MicroPathFinder.CancelCurrentPath();

                            // Pick up clean sheets
                            InteractWith(TargetCabinet);
                        }

                        // Bed no longer dirty, or is being interacted with by someone else
                        if ((TargetBed.CurrentState != "Dirty" || (TargetBed.TargetedBy != this) ||
                            (TargetBed.IsInteractingWith<Nurse>() && (TargetBed.GetInteractable<Nurse>() != this))))
                        {
                            SetIdle();
                        }

                        break;
                    }
                case Item.CleanSheets:
                    {
                        if (TargetBed != null)
                        {
                            // Move to bed if targeted
                            PathFindToPosition(TargetBed.Position);

                            // Close enough to interact with the bed
                            if (InteractionPoly.Intersects(ref TargetBed.BoundingPolygon))
                            {
                                MacroPathFinder.CancelCurrentPath();
                                MicroPathFinder.CancelCurrentPath();

                                // Clean the bed
                                InteractWith(mTargetBed);
                            }

                            // Bed no longer dirty, or being interacted with (but still have CleanSheets)
                            if ((TargetBed.CurrentState != "Dirty" && CurrentItem == Item.CleanSheets) ||
                                (TargetBed.IsInteractingWith<Nurse>() && (TargetBed.GetInteractable<Nurse>() != this)))
                            {
                                TargetCabinet = TargetBed.NearestCabLaundry;
                                TargetBed = null;
                            }
                        }

                        // Bed no longer being targeted, but still have sheets
                        if (TargetBed == null && CurrentItem == Item.CleanSheets)
                        {
                            // Cancel previous path to target bed
                            if (MacroPathFinder.TargetPosition2D != MacroPathFinder.GetCollisionMapNode(
                                new Vector2(TargetCabinet.Position.X, TargetCabinet.Position.Z)).Position)
                            {
                                MacroPathFinder.CancelCurrentPath();
                                MicroPathFinder.CancelCurrentPath();
                            }

                            // Move to nearest laundry cabinet
                            PathFindToPosition(TargetCabinet.Position);

                            // Close enough to interact with the cabinet
                            if (InteractionPoly.Intersects(ref TargetCabinet.BoundingPolygon))
                            {
                                MacroPathFinder.CancelCurrentPath();
                                MicroPathFinder.CancelCurrentPath();

                                // Return clean sheets
                                InteractWith(TargetCabinet);
                                SetIdle();
                            }
                        }
                        break;
                    }
                case Item.DirtySheets:
                    {
                        // Cancel previous path to target bed
                        if (MacroPathFinder.TargetPosition3D == TargetBed.Position)
                        {
                            MacroPathFinder.CancelCurrentPath();
                            MicroPathFinder.CancelCurrentPath();
                        }

                        // Move to nearest laundry cabinet
                        PathFindToPosition(TargetCabinet.Position);

                        // Close enough to interact with the bed
                        if (InteractionPoly.Intersects(ref TargetCabinet.BoundingPolygon))
                        {
                            MacroPathFinder.CancelCurrentPath();
                            MicroPathFinder.CancelCurrentPath();

                            // Put away dirty sheets
                            InteractWith(TargetBed.NearestCabLaundry);
                            SetIdle();
                        }

                        break;
                    }
            }
        }

        /// Entering the hospital at the beginning of a shift
        public void SetEnterHospital()
        {
            MacroPathFinder.CancelCurrentPath();
            MicroPathFinder.CancelCurrentPath();

            mGoal = AIGoal.EnterHospital;

            Rotation.Y = 180;
            //if (!IsAnimated)
            {
                SetAnimation("Run", 0.0f, 2.5f);
            }

            mNoAutoAnim = false;

        }
        private void UpdateEnterHospital()
        {
            if (Position.Z > 20)
            {
                Position.Z -= 0.2f;
            }
            else
            {
                if (IsAnimationPlaying && CurrentAnimationName != "Idle")
                {
                    SetAnimation("Idle", 0.0f, 1.0f);
                }

                SetIdle();
            }

            // WTF Karn this doesn't make any sense
            mElapsedMillisecs = 0;
        }

        /// Leaving the hospital at the end of the shift
        public void SetLeaveHospital()
        {
            MacroPathFinder.CancelCurrentPath();
            MicroPathFinder.CancelCurrentPath();

            // Find a path to the exit
            MacroPathFinder.AutoFollowPath = true;
            MacroPathFinder.FindPathTo(new Vector3(-2, 0, 20));

            // Just get rid of any items held
            CurrentItem = Item.None;

            if (TargetBed != null)
            {
                TargetBed.TargetedBy = null;
            }

            if (TargetCabinet != null)
            {
                TargetCabinet.TargetedBy = null;
            }

            if (TargetPatient != null)
            {
                TargetPatient.TargetedBy = null;
            }

            if (TargetChair != null)
            {
                TargetChair.TargetedBy = null;
            }

            mGoal = AIGoal.LeaveHospital;
        }
        private void UpdateLeaveHospital()
        {
            // If near exit, walk out
            if (MacroPathFinder.PathFollowed)
            {
                MacroPathFinder.AutoFollowPath = false;

                mNoAutoAnim = true;
                if (IsAnimationPlaying && CurrentAnimationName != "Run")
                {
                    SetAnimation("Run", 0.0f, 2.5f);
                }

                if (Position.Z < 60 - mSpawnPoint.Z)
                {
                    Rotation.Y = 0;
                    Position.Z += 0.2f;
                }
                else
                {
                    Active = false;
                    MacroPathFinder.CancelCurrentPath();
                    MicroPathFinder.CancelCurrentPath();
                }
            }

            mElapsedMillisecs = 0;
        }

        /// <summary>
        /// Creates a path to the specified position and follows it.
        /// </summary>
        /// <param name="position">Final destination</param>
        private void PathFindToPosition(Vector3 position)
        {
            // Find macro path
            if (!MacroPathFinder.FindingPath && !MacroPathFinder.HasPathBeenFound)
            {
                MacroPathFinder.AutoFollowPath = true;
                MacroPathFinder.FindPathTo(position);
                mMicroPathFindTime = 0;
            }

            // Find a micro path from the end of the macro path
            if (MacroPathFinder.HasPathBeenFound)
            {
                // Find a micro path                
                if (!MicroPathFinder.FindingPath && !MicroPathFinder.IsFollowingPath && !MicroPathFinder.HasPathBeenFound)
                {
                    MicroPathFinder.AutoFollowPath = false;
                    MicroPathFinder.FindPathFromTo(new Vector3(MacroPathFinder.LastNode.Position.X, 0,
                        MacroPathFinder.LastNode.Position.Y), position);
                }
                // Follow micro path after macro path end
                else if (MicroPathFinder.HasPathBeenFound && !MicroPathFinder.IsFollowingPath && !MicroPathFinder.FindingPath && !MacroPathFinder.IsFollowingPath)
                {
                    MicroPathFinder.FollowPath();
                }

            }

            // It's taking too long to find a proper path, jump straight to a micro one
            else if (mMicroPathFindTime >= mMicroPathFindMaxTime && !MicroPathFinder.HasPathBeenFound)
            {
                mMicroPathFindTime = 0;

                // Find a micro path form Nurse's current position
                if (!MacroPathFinder.HasPathBeenFound && !MicroPathFinder.HasPathBeenFound && !MicroPathFinder.FindingPath)
                {
                    MicroPathFinder.AutoFollowPath = true;
                    MicroPathFinder.FindPathFromTo(this.Position, position);
                }
            }
        }
    }
}